import glob
import time
import os
import re

def get_proc_list():
    proc_list = glob.glob("/proc/[0-9]*")
    proc_stat_list = []
    for i in range(0, len(proc_list)):
        proc_stat_list.append(proc_list[i] + "/stat")
    return proc_stat_list


def generate_proc_stat_fp(previous_proc_list, actual_proc_list, proc_stat_fp_dict):
    actual_proc_list_tmp = list(actual_proc_list)
    if (previous_proc_list != actual_proc_list):
        if (previous_proc_list is not None):
            for element in previous_proc_list:
                if element not in actual_proc_list:
                    proc_stat_fp_dict[element].close()
                    proc_stat_fp_dict.pop(element)
                    print element + " closed"

        for element in actual_proc_list_tmp:
            if (previous_proc_list is not None):
                if element not in previous_proc_list:
                    print element + " opened"
                    try:
                        proc_stat_fp_dict[element] = open(element)
                    except IOError:
                        #element does not exist anymore
                        actual_proc_list.remove(element)
                        print element + " not exist anymore"
                        pass
    return actual_proc_list


def get_cpu_tick_per_second():
    return os.sysconf('SC_CLK_TCK')

#Should use a class
def make_state_explicit(proc_stat_dict):
    if (proc_stat_dict is not None):
        dict_state = {
            'R': 'Running',
            'S': 'Sleeping in an interruptible wait',
            'D': 'Waiting in uninterruptible disk sleep',
            'Z': 'Zombie',
            'T': 'Stopped (on a signal) or (before Linux 2.6.33) trace stopped',
            't': 'Tracing stop',
            'W': 'Paging',
            'X': 'Dead',
            'x': 'Dead',
            'K': 'WakeKill',
            'W': 'Waking',
            'P': 'Parked'
        }

        try:
            state_value = proc_stat_dict['state']
        except KeyError: #this should NEVER happen
            proc_stat_dict['state'] = "Not valid"
            return None

        try:
            proc_stat_dict['state'] = dict_state[state_value]
        except:
            proc_stat_dict['state'] = "Not recognized state"



def get_provided_values_list():
    # http://man7.org/linux/man-pages/man5/proc.5.html
    values = ['pid', 'comm', 'state', 'ppid', 'pgrp', 'session', 'tty_nr', 'tpgid', 'flags', 'minflt', 'cminflt',
              'majflt', 'cmajflt', 'utime', 'stime', 'cutime', 'cstime', 'priority', 'nice', 'num_threads',
              'itrealvalue', 'starttime', 'vsize', 'rss', 'rsslim', 'startcode', 'endocode', 'startstack', 'kstkesp',
              'kstkeip', 'signal', 'blocked', 'sigignore', 'sigcatch', 'wchan', 'nswap', 'cnswap', 'exit_signal',
              'processor', 'rt_priority', 'policy', 'delayacct_blkio_ticks', 'guest_time', 'cguest_time', 'start_data',
              'end_data', 'start_brk', 'arg_start', 'arg_end', 'env_start', 'env_end', 'exit_code']
    return values


def print_proc_stat_dict(proc_stat_dict):
    if proc_stat_dict is not None:
        for element in get_provided_values_list():
            value = proc_stat_dict.get(element)
            if value:
                print element + ":" + value


def parse_process_stat(stat_file_fp):
    if (stat_file_fp):
        data_dict= {}
        data_names = get_provided_values_list()
        try:
            data = stat_file_fp.read()
            stat_file_fp.seek(0)
        except IOError:
            print("Does not exist")
            return None

        print data
        name = re.findall(" \(.*\) ", data)

        try:#ask forgiveness not permission
            name = name[0]
        except IndexError:
            name = "no_name"
            print "no name_found ??"


        # PLZ IMPROVE ME THIS IS HORRIBLE
        data_list = re.split('\(.*\)',data)
        data_list = re.split(' *',data_list[0])  + re.split(' *',data_list[1])
        data_list.insert(1, name)
        data_list = [x for x in data_list if x != '']

        data_names_counter = 0
        for data in data_list:
            data_name = data_names[data_names_counter]
            data_dict[data_name] = data
            data_names_counter += 1
        print data_dict
        return data_dict



def main():
    proc_fp_dict = {}
    previous_proc_list = []
    while True:
        proc_list = get_proc_list()
        previous_proc_list = generate_proc_stat_fp(previous_proc_list, proc_list, proc_fp_dict)

        for key,file_pointer in proc_fp_dict.iteritems():
            data_dict = parse_process_stat(file_pointer)
            make_state_explicit(data_dict)
            print_proc_stat_dict(data_dict)
        time.sleep(0.04)

main()





from plotly.graph_objs import Scatter, Figure, Layout
import dash
from dash.dependencies import Input, Output
import dash_core_components as dcc
import dash_html_components as html
import re


def get_log_fp(filename):
    return open(filename, "r")

def get_data_from_file(fp):
    chunk_list = []
    actual_chunk = {}
    actual_chunk_list = None
    data_names = ["name", "user", "nice", "system", "iowait", "irq","softirq", "idle"]
    while True: #sorry
        line = fp.readline()
        if (line == ""): #dirty sorry
            break

        if (line[0] == "["): #timestamp and New Chunk
            if (actual_chunk_list):
                chunk_list.append(actual_chunk_list)
            actual_chunk_list = []
            line = re.sub("[^0-9.]","", line)
            time_stamp = float(line)
            print time_stamp
            actual_chunk["timestamp"] = time_stamp
        else:
            temp_chunk = {}
            temp_chunk["timestamp"] = actual_chunk["timestamp"]
            actual_chunk = temp_chunk
            for data_name in data_names:
                actual_chunk[data_name] = extract_from_line(data_name, line)
            actual_chunk_list.append(actual_chunk)

    if (actual_chunk_list):
        chunk_list.append(actual_chunk_list)
    return chunk_list




def build_x_values(chunk_list_list):
    x_values = []
    for values_for_a_timestamp in chunk_list_list:
        x_values.append(values_for_a_timestamp[0].get("timestamp"))
    return x_values


def build_value_list(chunk_list_list, data_name):
    result_list = []
    if (len(chunk_list_list) >= 1):
        nb_cpu = len(chunk_list_list[0])
        for i in range (0, nb_cpu):
            result_list.append([])

    for values_for_a_timestamp in chunk_list_list:
        result_list_number = 0
        for data_for_a_cpu in values_for_a_timestamp:
            result_list[result_list_number].append(data_for_a_cpu[data_name])
            result_list_number += 1

    return result_list


def extract_from_line(data_name, line):
    regexp = ":[ ][a-z]*[0-9]*"
    data_to_match = data_name + regexp
    matching_result = re.search(data_to_match, line)
    extracted = matching_result.group().replace(data_name + ": ", "")
    return extracted


def get_name_list(chunk_list_list):
    cpu_name_list = []
    if (len(chunk_list_list) > 1):
        element = chunk_list_list[0]
        if (len(element) >= 1):
            for cpu in element:
                cpu_name_list.append(cpu["name"])
    return cpu_name_list

def build_option(cpu_names):
    option_list =[]
    for name in cpu_names:
        actual_dict = {}
        actual_dict["label"] = name
        actual_dict["value"] = name
        option_list.append(actual_dict)
    return option_list


def draw_graph(logfilename):
    chunk_list_list = get_data_from_file(get_log_fp(logfilename))# chunk_list_list =[[{name: "bla"...}, {name: "blo"...}], [{name: "bla"...}, {name: "blo"...}]]
    cpu_name_list = get_name_list(chunk_list_list)
    dash_options = build_option(cpu_name_list)
    x_values = build_x_values(chunk_list_list)

    data_available = ["user", "nice", "system", "iowait", "irq","softirq"]#, "idle"]
    data_dict = {}
    for element in data_available:
        data_dict[element] = build_value_list(chunk_list_list,element)

    nb_cpu = len(data_dict[element])
    data_per_cpu_list = [] #one dict per cpu, each dict contain data lists for this cpu#[{idle: [], user: [] ...} {}]
    for i in range(0, nb_cpu):
        actual_cpu_dict = {}
        for data in data_available:
            actual_cpu_dict[data] = data_dict[data][i]
        data_per_cpu_list.append(actual_cpu_dict)

    scatter_per_cpu_list = []

    for i in range(0, nb_cpu):
        actual_cpu_dict = []
        for data_name in data_available:
            actual_cpu_dict.append(Scatter(x = x_values, y = data_per_cpu_list[i][data_name], name = data_name))
        scatter_per_cpu_list.append(actual_cpu_dict)

    data_for_server = []
    for i in range(0, nb_cpu):
        actual_cpu_list = []
        for data_name in data_available:
            actual_data_dict = {}
            actual_data_dict["name"] = data_name
            actual_data_dict["x"] = x_values
            actual_data_dict["y"] = data_per_cpu_list[i][data_name]
            actual_cpu_list.append(actual_data_dict)
        data_for_server.append(actual_cpu_list)

    app = dash.Dash()

    app.layout = html.Div([
        html.H1('CPU-monitor'),
        dcc.Dropdown(
            id='my-dropdown',
            options=dash_options,
            value='cpu'
        ),
        dcc.Graph(id='my-graph')
    ])

    @app.callback(Output('my-graph', 'figure'), [Input('my-dropdown', 'value')])
    def update_graph(selected_dropdown_value):
        print selected_dropdown_value
        a = re.findall(r'\d+', selected_dropdown_value)
        index = 0
        if len(a) > 0:
            index = int(a[0]) + 1
        return {
            'data': data_for_server[index]
        }
    app.run_server()


from colorama import init
init()
from colorama import Back, Style, Fore
def pretty_print(cpu_stat):
    pretty = ""
    data = ["name", "user", "nice", "system", "iowait", "irq","softirq", "idle"]
    for element in data:
        value = cpu_stat.get(element)
        if value is not None:
            pretty += element +": " + str(value) + "  "
    return pretty

def create_colors():
    colors_dic = {}
    colors_dic["user"] = Back.RED
    colors_dic["nice"] = Back.CYAN
    colors_dic["system"] = Back.GREEN
    colors_dic["iowait"] = Back.YELLOW
    colors_dic["irq"] = Back.MAGENTA
    colors_dic["softirq"] = Back.BLUE
    colors_dic["idle"] = Back.WHITE
    return colors_dic

def print_legend():
    legend = ""
    color_dic = create_colors()
    for load_type, color in color_dic.iteritems():
        legend += load_type + ":" + color + " " + Style.RESET_ALL + " "
    print (legend)


def graphic_print(cpu_stat):
    colors_dic = create_colors()
    data = ["name","user", "nice", "system", "iowait", "irq","softirq", "idle"]
    percent_bar = ""
    size_of_num = 0
    for element in data:
        value = cpu_stat.get(element)
        if (element=="name"):
            print value

        elif value is not None:
            percent_bar_part = " "  * value
            color = colors_dic.get(element)
            if color:
                percent_bar_part = color  + percent_bar_part + Style.RESET_ALL
                percent_bar_part, size_of_num_tmp = insert_percent_value(percent_bar_part, value, color)
                size_of_num += size_of_num_tmp
                percent_bar += percent_bar_part

    percent_bar_size = percent_bar.count(" ") + size_of_num
    if (percent_bar_size < 100):
        percent_bar += colors_dic.get("idle") + " " * (100 - percent_bar_size) + Style.RESET_ALL
    print percent_bar


def insert_percent_value(percent_bar_part, value, color):
    if (value != 0):
        str_value = str(value)
        size_value = len(str_value)
        size_bar = percent_bar_part.count(" ")
        index_value = 0
        list_bar = list(percent_bar_part)
        for index_percent_bar in range(size_bar/2, size_bar/2 + size_value):
            list_bar[index_percent_bar + len(color)] = str_value[index_value]
            index_value += 1
        return "".join(list_bar), size_value
    else:
        return percent_bar_part, 0





import time
import re
import argparse
from cpu_info import get_cpu_info
from graphic_output import pretty_print, graphic_print, print_legend
from graph_maker import draw_graph
from loader import launch_cpu_load
import os

def pars_args():
    parser = argparse.ArgumentParser(description="A cpu usage logging software")
    parser.add_argument("--no_display", action="store_true", default=False, help="programm run without displaying and can"
                                                                              " still log")
    parser.add_argument("--version", action="version", version='%(prog)s 0.01')
    parser.add_argument("--load", action="store", dest="load_nb_threads", help="Define the number of threads to load during the monitoring"
                                                                           "If not used the cpu won't be loaded")
    parser.add_argument("--load_standalone", action="store_true", default=False, help="Does not monitor the CPU usage but "
                                                                                 "only launch load the "
                                                                                 "N threads provided by the --load "
                                                                                 "option, if --load option is not "
                                                                                 "provided only one thread will have "
                                                                                 "workload.")
    parser.add_argument("--display_operations_per_second", action="store_true", default=False, help="display number of "
                                                                                                 "operation per "
                                                                                                 "second if "
                                                                                                 "--loadstandalone "
                                                                                                 "option is used.")

    parser.add_argument("--cpu_info", action="store_true", default=False, help="Display informations about the CPU used and "
                                                                           "exit.")
    parser.add_argument("--cpu_info_detailed", action="store_true", default=False, help="Display extended informations "
                                                                                   "about the CPU used and exit.")
    parser.add_argument("--monitoring_freq", action="store", default=1, help="Monitoring frequency in Hz")
    parser.add_argument("--log_to_file", action="store", dest="filename", help="Log gathered results to a file")
    parser.add_argument("--graph_from_log", action="store", dest="logfile", help="Draw  graph for each core and one for the "
                                                                             "whole cpu from the LOGFILE argument")
    parser.add_argument("--fancy", action="store_true", default=False, help="Display fancy percent color bars during cpu "
                                                                        "monitoring.")

    parser.add_argument("--log_size", action="store", default=-1, help="Define the maximum number of lines to keep in the "
                                                                  "log file")

    return parser.parse_args()






def read_proc(proc_file):
    proc_content = proc_file.read()
    proc_file.seek(0)
    proc_lines = proc_content.split("\n")
    revelant_lines = []
    for line in proc_lines:
        if ("cpu" in line):
            revelant_lines.append(line)
    return revelant_lines

def calculate_percent(part, all):
    return (part * 100) / all

def calculator(actual_line_arg_str, previous_line_arg_str):
    actual_line_str = re.split("\s*", actual_line_arg_str)
    previous_line_str = re.split("\s*", previous_line_arg_str)
    name = actual_line_str[0]
    actual_line = []
    previous_line = []
    for i in range(1, len(actual_line_str)):
        previous_line.append(int(previous_line_str[i]))
        actual_line.append(int(actual_line_str[i]))

    actual_user = actual_line[0]
    actual_nice = actual_line[1]
    actual_system = actual_line[2]
    actual_idle = actual_line[3]
    actual_iowait = actual_line[4]
    actual_irq = actual_line[5]
    actual_softirq = actual_line[6]

    previous_user = previous_line[0]
    previous_nice = previous_line[1]
    previous_system = previous_line[2]
    previous_idle = previous_line[3]
    previous_iowait = previous_line[4]
    previous_irq = previous_line[5]
    previous_softirq = previous_line[6]

    #time per task type
    user = actual_user - previous_user
    nice = actual_nice - previous_nice
    system = actual_system - previous_system
    idle = actual_idle - previous_idle
    iowait = actual_iowait - previous_iowait
    irq = actual_irq - previous_irq
    softirq = actual_softirq - previous_softirq

    time_spent = user + nice + system + idle + iowait + irq + softirq

    relevant = True
    if (time_spent == 0):
        relevant = False

    if(relevant):
        result = {}
        user_percent = calculate_percent(user, time_spent)
        nice_percent = calculate_percent(nice, time_spent)
        system_percent = calculate_percent(system, time_spent)
        idle_percent = calculate_percent(idle,time_spent)
        iowait_percent = calculate_percent(iowait, time_spent)
        irq_percent = calculate_percent(irq, time_spent)
        softirq_percent = calculate_percent(softirq, time_spent)
        result["user"] = user_percent
        result["nice"] = nice_percent
        result["system"] = system_percent
        result["idle"] = idle_percent
        result["iowait"] = iowait_percent
        result["irq"] = irq_percent
        result["softirq"] = softirq_percent
        result["name"] = name
        return result
    else:
        return None


def set_time_stamp(actual_log_size, log_file):
    if(log_file):
        time_stamp = "[ " + str(time.time()) + " ]\n"
        log_file.write(time_stamp)
        actual_log_size += 1
    return actual_log_size

def write_to_log(actual_log_size, max_logsize, value_to_write, log_fp):
    if(value_to_write is not None and log_fp):
        log_fp.write(str(value_to_write) + "\n")
        actual_log_size += 1
    return actual_log_size

def log(log_file_pointer, max_log_file_size, update_freq, fancy, proc_file, no_display):
    sleep_time = 1 / float(update_freq)
    previous_lines = None
    actual_log_size = 0
    while True:
        revelant_lines = read_proc(proc_file)
        if (previous_lines and previous_lines != revelant_lines):
            actual_log_size = set_time_stamp(actual_log_size, log_file_pointer)
            for i in range(0, len(revelant_lines)):
                result = calculator(revelant_lines[i], previous_lines[i])
                to_print_or_save = pretty_print(result)
                if(no_display == False):
                    if (fancy):
                        graphic_print(result)
                    else:
                        print to_print_or_save
                actual_log_size = write_to_log(actual_log_size,max_log_file_size,
                                                                 to_print_or_save, log_file_pointer)
        previous_lines = revelant_lines
        time.sleep(sleep_time)
        os.system('clear')
        if fancy:
            print_legend()
        if log_file_pointer is not None:
            log_file_pointer.flush()
            if actual_log_size >= max_log_file_size and max_log_file_size > -1:
                log_file_pointer.seek(0)
                print actual_log_size
                actual_log_size = 0


def main():
    parsed_args = pars_args()
    display = parsed_args.display_operations_per_second
    standalone = parsed_args.load_standalone
    max_log_size = int(parsed_args.log_size)
    #display values
    fancy = parsed_args.fancy
    no_display = parsed_args.no_display

    freq = int(parsed_args.monitoring_freq)
    filename = parsed_args.filename
    proc_file = open("/proc/stat", "r")
    fp = None
    if filename:
        fp = open(filename, 'w')  # not closed !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    if(parsed_args.logfile):
        draw_graph(parsed_args.logfile)
        exit(0)
    if(parsed_args.load_nb_threads):
        load = int(parsed_args.load_nb_threads)
    else:
        load = None
    if (parsed_args.cpu_info):
        get_cpu_info(to_display=True)
        exit()
    if (parsed_args.cpu_info_detailed):
        get_cpu_info(to_display=True, detailed=True)
        exit()
    if (display):
        if (not standalone):
            display = False
    if (load is not None or standalone):
        if (load is not None):
            launch_cpu_load(display, standalone, no_of_cpu_to_be_consumed=load)
        else:
            launch_cpu_load(display, standalone)
        if standalone:
            while True:
                time.sleep(1)
    log(fp, max_log_size, freq, fancy, proc_file, no_display)

if __name__ == '__main__':
    main()

main()
